@@ -6,6 +6,13 @@ use std::result;
 
 use bstr::{BString, ByteSlice};
 
+#[cfg(feature = "with_serde")]
+use serde::de::Deserialize;
+
+#[cfg(feature = "with_serde")]
+use crate::deserializer::deserialize_byte_record;
+#[cfg(feature = "with_serde")]
+use crate::error::Result;
 use crate::error::{new_utf8_error, Utf8Error};
 use crate::string_record::StringRecord;
 
@@ -468,6 +475,98 @@ impl ByteRecord {
             }
         }
     }
+    /// Deserialize this record.
+    ///
+    /// The `D` type parameter refers to the type that this record should be
+    /// deserialized into. The `'de` lifetime refers to the lifetime of the
+    /// `ByteRecord`. The `'de` lifetime permits deserializing into structs
+    /// that borrow field data from this record.
+    ///
+    /// An optional `headers` parameter permits deserializing into a struct
+    /// based on its field names (corresponding to header values) rather than
+    /// the order in which the fields are defined.
+    ///
+    /// # Example: without headers
+    ///
+    /// This shows how to deserialize a single row into a struct based on the
+    /// order in which fields occur. This example also shows how to borrow
+    /// fields from the `ByteRecord`, which results in zero allocation
+    /// deserialization.
+    ///
+    /// ```
+    /// use std::error::Error;
+    ///
+    /// use csv_async::ByteRecord;
+    /// use serde::Deserialize;
+    ///
+    /// #[derive(Deserialize)]
+    /// struct Row<'a> {
+    ///     city: &'a str,
+    ///     country: &'a str,
+    ///     population: u64,
+    /// }
+    ///
+    /// # fn main() { example().unwrap() }
+    /// fn example() -> Result<(), Box<dyn Error>> {
+    ///     let record = ByteRecord::from(vec![
+    ///         "Boston", "United States", "4628910",
+    ///     ]);
+    ///
+    ///     let row: Row = record.deserialize(None)?;
+    ///     assert_eq!(row.city, "Boston");
+    ///     assert_eq!(row.country, "United States");
+    ///     assert_eq!(row.population, 4628910);
+    ///     Ok(())
+    /// }
+    /// ```
+    ///
+    /// # Example: with headers
+    ///
+    /// This example is like the previous one, but shows how to deserialize
+    /// into a struct based on the struct's field names. For this to work,
+    /// you must provide a header row.
+    ///
+    /// This example also shows that you can deserialize into owned data
+    /// types (e.g., `String`) instead of borrowed data types (e.g., `&str`).
+    ///
+    /// ```
+    /// use std::error::Error;
+    ///
+    /// use csv_async::ByteRecord;
+    /// use serde::Deserialize;
+    ///
+    /// #[derive(Deserialize)]
+    /// struct Row {
+    ///     city: String,
+    ///     country: String,
+    ///     population: u64,
+    /// }
+    ///
+    /// # fn main() { example().unwrap() }
+    /// fn example() -> Result<(), Box<dyn Error>> {
+    ///     // Notice that the fields are not in the same order
+    ///     // as the fields in the struct!
+    ///     let header = ByteRecord::from(vec![
+    ///         "country", "city", "population",
+    ///     ]);
+    ///     let record = ByteRecord::from(vec![
+    ///         "United States", "Boston", "4628910",
+    ///     ]);
+    ///
+    ///     let row: Row = record.deserialize(Some(&header))?;
+    ///     assert_eq!(row.city, "Boston");
+    ///     assert_eq!(row.country, "United States");
+    ///     assert_eq!(row.population, 4628910);
+    ///     Ok(())
+    /// }
+    /// ```
+    #[cfg(feature = "with_serde")]
+    pub fn deserialize<'de, D: Deserialize<'de>>(
+        &'de self,
+        headers: Option<&'de ByteRecord>,
+    ) -> Result<D> {
+        deserialize_byte_record(self, headers)
+    }
 }
 
 /// A position in CSV data.
@@ -667,6 +766,7 @@ impl<T: AsRef<[u8]>> Extend<T> for ByteRecord {
 ///
 /// The `'r` lifetime variable refers to the lifetime of the `ByteRecord` that
 /// is being iterated over.
+#[derive(Clone)]
 pub struct ByteRecordIter<'r> {
     /// The record we are iterating over.
     r: &'r ByteRecord,
